@@ -23,6 +23,7 @@
 
 $Id: BenchRunner.py 24746 2005-08-31 09:59:27Z bdelbosc $
 """
+from __future__ import absolute_import
 import os
 import platform
 import sys
@@ -35,32 +36,48 @@ from optparse import OptionParser, TitledHelpFormatter
 from socket import error as SocketError
 from thread import error as ThreadError
 from xmlrpclib import ServerProxy, Fault
+import signal
 
-from FunkLoadHTTPServer import FunkLoadHTTPServer
-from utils import mmn_encode, set_recording_flag, recording, thread_sleep, \
+from .FunkLoadTestCase import FunkLoadTestCase
+from .FunkLoadHTTPServer import FunkLoadHTTPServer
+from .utils import mmn_encode, set_recording_flag, recording, thread_sleep, \
                   trace, red_str, green_str, get_version
+try:
+    from funkload.rtfeedback import (FeedbackSender, DEFAULT_ENDPOINT,
+                                     DEFAULT_PUBSUB)
+    LIVE_FEEDBACK = True
+except ImportError:
+    LIVE_FEEDBACK = False
+    DEFAULT_PUBSUB = DEFAULT_ENDPOINT = None
 
 
 USAGE = """%prog [options] file class.method
 
 %prog launch a FunkLoad unit test as load test.
 
-A FunkLoad unittest use a configuration file named [class].conf, this
-configuration is overriden by the command line options.
+A FunkLoad unittest uses a configuration file named [class].conf. This
+configuration may be overriden by the command line options.
 
 See http://funkload.nuxeo.org/ for more information.
 
 Examples
 ========
   %prog myFile.py MyTestCase.testSomething
+  %prog my_module MyTestCase.testSomething
                         Bench MyTestCase.testSomething using MyTestCase.conf.
   %prog -u http://localhost:8080 -c 10:20 -D 30 myFile.py \\
       MyTestCase.testSomething
                         Bench MyTestCase.testSomething on localhost:8080
-                        with 2 cycles of 10 and 20 users during 30s.
+                        with 2 cycles of 10 and 20 users for a duration of 30s.
   %prog -h
                         More options.
+
+Alternative Usage:
+  %prog discover [options]
+                        Discover test modules in the current directory and
+                        bench all of them.
 """
+
 try:
     import psyco
     psyco.full()
@@ -88,6 +105,8 @@ def add_cycle_result(status):
     else:
         g_failures += 1
 
+    return g_success, g_errors, g_failures
+
 
 def get_cycle_results():
     """Return counters."""
@@ -121,9 +140,18 @@ def reset_cycle_results():
     g_success = g_failures = g_errors = 0
 
 
+def load_module(test_module):
+    module = __import__(test_module)
+    parts = test_module.split('.')[1:]
+    while parts:
+        part = parts.pop()
+        module = getattr(module, part)
+    return module
+
+
 def load_unittest(test_module, test_class, test_name, options):
     """instantiate a unittest."""
-    module = __import__(test_module)
+    module = load_module(test_module)
     klass = getattr(module, test_class)
     return klass(test_name, options)
 
@@ -158,18 +186,22 @@ class LoopTestRunner(threading.Thread):
 
     def __init__(self, test_module, test_class, test_name, options,
                  cycle, cvus, thread_id, thread_signaller, sleep_time,
-                 debug=False):
+                 debug=False, feedback=None):
         meta_method_name = mmn_encode(test_name, cycle, cvus, thread_id)
         threading.Thread.__init__(self, target=self.run, name=meta_method_name,
                                   args=())
         self.test = load_unittest(test_module, test_class, meta_method_name,
                                   options)
-        self.color = not options.no_color
+        if sys.platform.lower().startswith('win'):
+            self.color = False
+        else:
+            self.color = not options.no_color
         self.sleep_time = sleep_time
         self.debug = debug
         self.thread_signaller = thread_signaller
         # this makes threads endings if main stop with a KeyboardInterupt
         self.setDaemon(1)
+        self.feedback = feedback
 
     def run(self):
         """Run a test in loop."""
@@ -177,41 +209,61 @@ class LoopTestRunner(threading.Thread):
             test_result = unittest.TestResult()
             self.test.clearContext()
             self.test(test_result)
+            feedback = {}
+
             if test_result.wasSuccessful():
                 if recording():
-                    add_cycle_result('success')
-                    if self.color:
-                        trace(green_str('.'))
-                    else:
-                        trace('.')
+                    feedback['count'] = add_cycle_result('success')
+
+                if self.color:
+                    trace(green_str('.'))
+                else:
+                    trace('.')
+
+                feedback['result'] = 'success'
             else:
                 if len(test_result.errors):
                     if recording():
-                        add_cycle_result('error')
+                        feedback['count'] = add_cycle_result('error')
+
                     if self.color:
                         trace(red_str('E'))
                     else:
                         trace('E')
+
+                    feedback['result'] = 'error'
+
                 else:
                     if recording():
-                        add_cycle_result('failure')
+                        feedback['count'] = add_cycle_result('failure')
+
                     if self.color:
                         trace(red_str('F'))
                     else:
                         trace('F')
+
+                    feedback['result'] = 'failure'
+
                 if self.debug:
+                    feedback['errors'] = test_result.errors
+                    feedback['failures'] = test_result.failures
+
                     for (test, error) in test_result.errors:
                         trace("ERROR %s: %s" % (str(test), str(error)))
                     for (test, error) in test_result.failures:
                         trace("FAILURE %s: %s" % (str(test), str(error)))
+
+            if self.feedback is not None:
+                self.feedback.test_done(feedback)
+
             thread_sleep(self.sleep_time)
 
 
 class BenchRunner:
     """Run a unit test in bench mode."""
 
-    def __init__(self, module_file, class_name, method_name, options):
-        self.module_name = os.path.basename(os.path.splitext(module_file)[0])
+    def __init__(self, module_name, class_name, method_name, options):
+        self.module_name = module_name
         self.class_name = class_name
         self.method_name = method_name
         self.options = options
@@ -243,18 +295,28 @@ class BenchRunner:
         if not options.is_distributed:
             hosts = test.conf_get('monitor', 'hosts', '', quiet=True).split()
             for host in hosts:
-                host = host.strip()
-                monitor_hosts.append((host, test.conf_getInt(host, 'port'),
-                                      test.conf_get(host, 'description', '')))
+                name = host
+                host = test.conf_get(host,'host',host.strip())
+                monitor_hosts.append((name, host, test.conf_getInt(name, 'port'),
+                                      test.conf_get(name, 'description', '')))
         self.monitor_hosts = monitor_hosts
         # keep the test to use the result logger for monitoring
         # and call setUp/tearDown Cycle
         self.test = test
 
+        # set up the feedback sender
+        if LIVE_FEEDBACK and options.is_distributed and options.feedback:
+            trace("* Creating Feedback sender")
+            self.feedback = FeedbackSender(endpoint=options.feedback_endpoint or
+                                           DEFAULT_ENDPOINT)
+        else:
+            self.feedback = None
+
     def run(self):
         """Run all the cycles.
 
         return 0 on success, 1 if there were some failures and -1 on errors."""
+
         trace(str(self))
         trace("Benching\n")
         trace("========\n\n")
@@ -278,7 +340,7 @@ class BenchRunner:
             trace(' done.\n')
             self.startMonitors(monitor_key)
             self.startThreads(cycle, cvus)
-            self.logging()
+            self.logging(cycle, cvus)
             #self.dumpThreads()
             self.stopThreads()
             self.stopMonitors(monitor_key)
@@ -356,7 +418,8 @@ class BenchRunner:
                                     self.method_name, self.options,
                                     cycle, number_of_threads,
                                     thread_id, thread_signaller,
-                                    self.sleep_time)
+                                    self.sleep_time,
+                                    feedback=self.feedback)
             trace(".")
             try:
                 thread.start()
@@ -371,13 +434,17 @@ class BenchRunner:
         trace(' done.\n')
         return threads
 
-    def logging(self):
+    def logging(self, cycle, cvus):
         """Log activity during duration."""
         duration = self.duration
         end_time = time.time() + duration
+        mid_time = time.time() + duration / 2
         trace("* Logging for %ds (until %s): " % (
             duration, datetime.fromtimestamp(end_time).isoformat()))
         set_recording_flag(True)
+        while time.time() < mid_time:
+            time.sleep(1)
+        self.test.midCycle(cycle, cvus)
         while time.time() < end_time:
             # wait
             time.sleep(1)
@@ -458,25 +525,25 @@ class BenchRunner:
         if not self.monitor_hosts:
             return
         monitor_hosts = []
-        for (host, port, desc) in self.monitor_hosts:
-            trace("* Getting monitoring config from %s: ..." % host)
+        for (name, host, port, desc) in self.monitor_hosts:
+            trace("* Getting monitoring config from %s: ..." % name)
             server = ServerProxy("http://%s:%s" % (host, port))
             try:
                 config = server.getMonitorsConfig()
                 data = []
                 for key in config.keys():
                     xml = '<monitorconfig host="%s" key="%s" value="%s" />' % (
-                                                        host, key, config[key])
+                                                        name, key, config[key])
                     data.append(xml)
                 self.logr("\n".join(data))
             except Fault:
                 trace(' not supported.\n')
-                monitor_hosts.append((host, port, desc))
+                monitor_hosts.append((name, host, port, desc))
             except SocketError:
                 trace(' failed, server is down.\n')
             else:
                 trace(' done.\n')
-                monitor_hosts.append((host, port, desc))
+                monitor_hosts.append((name, host, port, desc))
         self.monitor_hosts = monitor_hosts
 
     def startMonitors(self, monitor_key):
@@ -484,8 +551,8 @@ class BenchRunner:
         if not self.monitor_hosts:
             return
         monitor_hosts = []
-        for (host, port, desc) in self.monitor_hosts:
-            trace("* Start monitoring %s: ..." % host)
+        for (name, host, port, desc) in self.monitor_hosts:
+            trace("* Start monitoring %s: ..." % name)
             server = ServerProxy("http://%s:%s" % (host, port))
             try:
                 server.startRecord(monitor_key)
@@ -493,15 +560,15 @@ class BenchRunner:
                 trace(' failed, server is down.\n')
             else:
                 trace(' done.\n')
-                monitor_hosts.append((host, port, desc))
+                monitor_hosts.append((name, host, port, desc))
         self.monitor_hosts = monitor_hosts
 
     def stopMonitors(self, monitor_key):
         """Stop monitoring and save xml result."""
         if not self.monitor_hosts:
             return
-        for (host, port, desc) in self.monitor_hosts:
-            trace('* Stop monitoring %s: ' % host)
+        for (name, host, port, desc) in self.monitor_hosts:
+            trace('* Stop monitoring %s: ' % name)
             server = ServerProxy("http://%s:%s" % (host, port))
             try:
                 server.stopRecord(monitor_key)
@@ -540,13 +607,14 @@ class BenchRunner:
         if self.options.label:
             config['label'] = self.options.label
 
-        for (host, port, desc) in self.monitor_hosts:
-            config[host] = desc
+        for (name, host, port, desc) in self.monitor_hosts:
+            config[name] = desc
         self.test._open_result_log(**config)
 
     def logr_close(self):
         """Stop logging tag."""
         self.test._close_result_log()
+        self.test.logger_result.handlers = []
 
     def __repr__(self):
         """Display bench information."""
@@ -573,14 +641,205 @@ class BenchRunner:
         return '\n'.join(text)
 
 
-def main():
-    """Default main."""
-    # enable to load module in the current path
-    cur_path = os.path.abspath(os.path.curdir)
-    sys.path.insert(0, cur_path)
+class BenchLoader(unittest.TestLoader):
+    suiteClass = list
+    def loadTestsFromTestCase(self, testCaseClass):
+        if not issubclass(testCaseClass, FunkLoadTestCase):
+            trace(red_str("Skipping "+ testCaseClass))
+            return []
+        testCaseNames = self.getTestCaseNames(testCaseClass)
+        if not testCaseNames and hasattr(testCaseClass, 'runTest'):
+            testCaseNames = ['runTest']
 
+        return [dict(module_name = testCaseClass.__module__,
+                     class_name = testCaseClass.__name__,
+                     method_name = x)
+                for x in testCaseNames]
+
+def discover(sys_args):
+    parser = get_shared_OptionParser()
+    options, args = parser.parse_args(sys_args)
+    options.label = None
+
+    loader = BenchLoader()
+    suite = loader.discover('.')
+
+    def flatten_test_suite(suite):
+        if type(suite) != BenchLoader.suiteClass:
+            # Wasn't a TestSuite - must have been a Test
+            return [suite]
+        flat = []
+        for x in suite:
+            flat += flatten_test_suite(x)
+        return flat
+
+    flattened = flatten_test_suite(suite)
+    retval = 0
+    for test in flattened:
+        module_name = test['module_name']
+        class_name = test['class_name']
+        method_name = test['method_name']
+        if options.distribute:
+            dist_args = sys_args[:]
+            dist_args.append(module_name)
+            dist_args.append('%s.%s' % (class_name, method_name))
+            ret = run_distributed(options, module_name, class_name,
+                                   method_name, dist_args)
+        else:
+            ret = run_local(options, module_name, class_name, method_name)
+        # Handle failures
+        if ret != 0:
+            retval = ret
+            if options.failfast:
+                break
+    return retval
+
+_manager = None
+
+def shutdown(*args):
+    trace('Aborting run...')
+    if _manager is not None:
+        _manager.abort()
+    trace('Aborted')
+    sys.exit(0)
+
+def get_runner_class(class_path):
+    try:
+        module_path, class_name = class_path.rsplit('.', 1)
+    except ValueError:
+        raise Exception('Invalid class path {0}'.format(class_path))
+
+    _module = __import__(module_path, globals(), locals(), class_name, -1)
+    return getattr(_module, class_name)
+
+def parse_sys_args(sys_args):
+    parser = get_shared_OptionParser()
+    parser.add_option("", "--config",
+                      type="string",
+                      dest="config",
+                      metavar='CONFIG',
+                      help="Path to alternative config file")
+    parser.add_option("-l", "--label",
+                      type="string",
+                      help="Add a label to this bench run for easier "
+                           "identification (it will be appended to the "
+                           "directory name for reports generated from it).")
+
+    options, args = parser.parse_args(sys_args)
+
+    if len(args) != 2:
+        parser.error("incorrect number of arguments")
+
+    if not args[1].count('.'):
+        parser.error("invalid argument; should be [class].[method]")
+
+    if options.as_fast_as_possible:
+        options.bench_sleep_time_min = '0'
+        options.bench_sleep_time_max = '0'
+        options.bench_sleep_time = '0'
+
+    if os.path.exists(args[0]):
+        # We were passed a file for the first argument
+        module_name = os.path.basename(os.path.splitext(args[0])[0])
+    else:
+        # We were passed a module name
+        module_name = args[0]
+
+    return options, args, module_name
+
+def get_shared_OptionParser():
+    '''Make an OptionParser that can be used in both normal mode and in
+    discover mode.
+    '''
     parser = OptionParser(USAGE, formatter=TitledHelpFormatter(),
                           version="FunkLoad %s" % get_version())
+    parser.add_option("-r", "--runner-class",
+                      type="string",
+                      dest="bench_runner_class",
+                      default="funkload.BenchRunner.BenchRunner",
+                      help="Python dotted import path to BenchRunner class to use.")
+    parser.add_option("", "--no-color",
+                      action="store_true",
+                      help="Monochrome output.")
+    parser.add_option("", "--accept-invalid-links",
+                      action="store_true",
+                      help="Do not fail if css/image links are not reachable.")
+    parser.add_option("", "--simple-fetch",
+                      action="store_true",
+                      dest="bench_simple_fetch",
+                      help="Don't load additional links like css or images "
+                           "when fetching an html page.")
+    parser.add_option("--enable-debug-server",
+                      action="store_true",
+                      dest="debugserver",
+                      help="Instantiates a debug HTTP server which exposes an "
+                           "interface using which parameters can be modified "
+                           "at run-time. Currently supported parameters: "
+                           "/cvu?inc=<integer> to increase the number of "
+                           "CVUs, /cvu?dec=<integer> to decrease the number "
+                           "of CVUs, /getcvu returns number of CVUs ")
+    parser.add_option("--debug-server-port",
+                      type="string",
+                      dest="debugport",
+                      help="Port at which debug server should run during the "
+                           "test")
+    parser.add_option("--distribute",
+                      action="store_true",
+                      dest="distribute",
+                      help="Distributes the CVUs over a group of worker "
+                           "machines that are defined in the workers section")
+    parser.add_option("--distribute-workers",
+                      type="string",
+                      dest="workerlist",
+                      help="This parameter will  override the list of "
+                           "workers defined in the config file. expected "
+                           "notation is uname@host,uname:pwd@host or just "
+                           "host...")
+    parser.add_option("--distribute-python",
+                      type="string",
+                      dest="python_bin",
+                      help="When running in distributed mode, this Python "
+                           "binary will be used across all hosts.")
+    parser.add_option("--is-distributed",
+                      action="store_true",
+                      dest="is_distributed",
+                      help="This parameter is for internal use only. It "
+                           "signals to a worker node that it is in "
+                           "distributed mode and shouldn't perform certain "
+                           "actions.")
+    parser.add_option("--distributed-packages",
+                      type="string",
+                      dest="distributed_packages",
+                      help="Additional packages to be passed to easy_install "
+                           "on remote machines when being run in distributed "
+                           "mode.")
+    parser.add_option("--distributed-log-path",
+                      type="string",
+                      dest="distributed_log_path",
+                      help="Path where all the logs will be stored when "
+                           "running a distributed test")
+    parser.add_option("--distributed-key-filename",
+                      type="string",
+                      dest="distributed_key_filename",
+                      help=("Path of the SSH key to use when running a "
+                            "distributed test"))
+    parser.add_option("--feedback-endpoint",
+                      type="string",
+                      dest="feedback_endpoint",
+                      help=("ZMQ push/pull socket used between the master and "
+                            "the node to send feedback."))
+    parser.add_option("--feedback-pubsub-endpoint",
+                      type="string",
+                      dest="feedback_pubsub_endpoint",
+                      help="ZMQ pub/sub socket use to publish feedback.")
+    parser.add_option("--feedback",
+                      action="store_true",
+                      dest="feedback",
+                      help="Activates the realtime feedback")
+    parser.add_option("--failfast",
+                      action="store_true",
+                      dest="failfast",
+                      help="Stop on first fail or error. (For discover mode)")
     parser.add_option("-u", "--url",
                       type="string",
                       dest="main_url",
@@ -588,9 +847,9 @@ def main():
     parser.add_option("-c", "--cycles",
                       type="string",
                       dest="bench_cycles",
-                      help="Cycles to bench, this is a list of number of "
-                           "virtual concurrent users, to run a bench with 3"
-                           "cycles with 5, 10 and 20 users use: -c 2:10:20")
+                      help="Cycles to bench, colon-separated list of "
+                           "virtual concurrent users. To run a bench with 3 "
+                           "cycles of 5, 10 and 20 users, use: -c 5:10:20")
     parser.add_option("-D", "--duration",
                       type="string",
                       dest="bench_duration",
@@ -615,116 +874,75 @@ def main():
                       action="store_true",
                       help="Remove sleep times between requests and between "
                            "tests, shortcut for -m0 -M0 -t0")
-    parser.add_option("", "--no-color",
-                      action="store_true",
-                      help="Monochrome output.")
-    parser.add_option("", "--accept-invalid-links",
-                      action="store_true",
-                      help="Do not fail if css/image links are not reachable.")
-    parser.add_option("", "--simple-fetch",
-                      action="store_true",
-                      dest="bench_simple_fetch",
-                      help="Don't load additional links like css or images "
-                           "when fetching an html page.")
-    parser.add_option("-l", "--label",
-                      type="string",
-                      help="Add a label to this bench run for easier "
-                           "identification (it will be appended to the "
-                           "directory name for reports generated from it).")
-    parser.add_option("--enable-debug-server",
-                      action="store_true",
-                      dest="debugserver",
-                      help="Instantiates a debug HTTP server which exposes an "
-                           "interface using which parameters can be modified "
-                           "at run-time. Currently supported parameters: "
-                           "/cvu?inc=<integer> to increase the number of "
-                           "CVUs, /cvu?dec=<integer> to decrease the number "
-                           "of CVUs, /getcvu returns number of CVUs ")
-    parser.add_option("--debug-server-port",
-                      type="string",
-                      dest="debugport",
-                      help="Port at which debug server should run during the "
-                           "test")
-    parser.add_option("--distribute",
-                      action="store_true",
-                      dest="distribute",
-                      help="Distributes the CVUs over a group of worker "
-                           "machines that are defined in the workers section")
-    parser.add_option("--distribute-workers",
-                      type="string",
-                      dest="workerlist",
-                      help="This parameter will  over-ride the list of "
-                           "workers defined in the config file. expected "
-                           "notation is uname@host,uname:pwd@host or just "
-                           "host...")
-    parser.add_option("--distribute-python",
-                      type="string",
-                      dest="python_bin",
-                      help="When running in distributed mode, this Python "
-                           "binary will be used across all hosts.")
-    parser.add_option("--is-distributed",
-                      action="store_true",
-                      dest="is_distributed",
-                      help="This parameter is for internal use only. it "
-                           "signals to a worker node that it is in "
-                           "distributed mode and shouldn't perform certain "
-                           "actions.")
-    parser.add_option("--distributed-packages",
-                      type="string",
-                      dest="distributed_packages",
-                      help="Additional packages to be passed to easy_install "
-                           "on remote machines when being run in distributed "
-                           "mode.")
-
-    options, args = parser.parse_args()
-    # XXX What exactly is this checking for here??
-    cmd_args = " ".join([k for k in sys.argv[1:]
-                           if k.find('--distribute') < 0])
-
-    if len(args) != 2:
-        parser.error("incorrect number of arguments")
-
-    if not args[1].count('.'):
-        parser.error("invalid argument; should be [class].[method]")
-
-    if options.as_fast_as_possible:
-        options.bench_sleep_time_min = '0'
-        options.bench_sleep_time_max = '0'
-        options.bench_sleep_time = '0'
-
-    klass, method = args[1].split('.')
-    if options.distribute:
-        from Distributed import DistributionMgr
-        ret = None
-        try:
-            distmgr = DistributionMgr(
-                args[0], klass, method, options, cmd_args)
-        except UserWarning, error:
-            trace(red_str("Distribution failed with:%s \n" % (error)))
+    return parser
 
+def run_distributed(options, module_name, class_name, method_name, sys_args):
+    ret = None
+    from funkload.Distributed import DistributionMgr
+    global _manager
+    
+    try:
+        distmgr = DistributionMgr(
+            module_name, class_name, method_name, options, sys_args)
+        _manager = distmgr
+    except UserWarning as error:
+        trace(red_str("Distribution failed with:%s \n" % (error)))
+        return 1
+    
+    try:
         try:
             distmgr.prepare_workers(allow_errors=True)
             ret = distmgr.run()
             distmgr.final_collect()
         except KeyboardInterrupt:
             trace("* ^C received *")
-            distmgr.abort()
+    finally:
+        # in any case we want to stop the workers at the end
+        distmgr.abort()
+    
+    _manager = None
+    return ret
 
-        sys.exit(ret)
+def run_local(options, module_name, class_name, method_name):
+    ret = None
+    RunnerClass = get_runner_class(options.bench_runner_class)
+    bench = RunnerClass(module_name, class_name, method_name, options)
+    
+    # Start a HTTP server optionally
+    if options.debugserver:
+        http_server_thread = FunkLoadHTTPServer(bench, options.debugport)
+        http_server_thread.start()
+    
+    try:
+        ret = bench.run()
+    except KeyboardInterrupt:
+        trace("* ^C received *")
+    return ret
+
+def main(sys_args=sys.argv[1:]):
+    """Default main."""
+    # enable loading of modules in the current path
+    cur_path = os.path.abspath(os.path.curdir)
+    sys.path.insert(0, cur_path)
+
+    # registering signals
+    if not sys.platform.lower().startswith('win'):
+        signal.signal(signal.SIGTERM, shutdown)
+        signal.signal(signal.SIGINT, shutdown)
+        signal.signal(signal.SIGQUIT, shutdown)
+
+    # special case: 'discover' argument
+    if sys_args and sys_args[0].lower() == 'discover':
+        return discover(sys_args)
+
+    options, args, module_name = parse_sys_args(sys_args)
+
+    klass, method = args[1].split('.')
+    if options.distribute:
+        return run_distributed(options, module_name, klass, method, sys_args)
     else:
-        bench = BenchRunner(args[0], klass, method, options)
-
-        # Start a HTTP server optionally
-        if options.debugserver == True:
-            http_server_thread = FunkLoadHTTPServer(bench, options.debugport)
-            http_server_thread.start()
-
-        ret = None
-        try:
-            ret = bench.run()
-        except KeyboardInterrupt:
-            trace("* ^C received *")
-        sys.exit(ret)
+        return run_local(options, module_name, klass, method)
 
 if __name__ == '__main__':
-    main()
+    ret = main()
+    sys.exit(ret)
