@@ -32,6 +32,8 @@
 
 $Id: PatchWebunit.py 24649 2005-08-29 14:20:19Z bdelbosc $
 """
+from __future__ import print_function
+from __future__ import absolute_import
 import os
 import sys
 import time
@@ -40,6 +42,8 @@ from urllib import urlencode
 import httplib
 import cStringIO
 from mimetypes import guess_type
+import datetime
+import Cookie
 
 from webunit import cookie
 from webunit.IMGSucker import IMGSucker
@@ -47,7 +51,7 @@ from webunit.webunittest import WebTestCase, WebFetcher
 from webunit.webunittest import HTTPResponse, HTTPError, VERBOSE
 from webunit.utility import Upload
 
-from utils import thread_sleep, Data
+from .utils import thread_sleep, Data
 import re
 
 valid_url = re.compile(r'^(http|https)://[a-z0-9\.\-\:]+(\/[^\ \t\<\>]*)?$',
@@ -113,7 +117,7 @@ class FKLIMGSucker(IMGSucker):
                     continue
                 # TODO: figure the re-write path
                 # newattributes.append((name, path))
-                if not self.session.images.has_key(url):
+                if url not in self.session.images:
                     self.ftestcase.logdd('    img: %s ...' % url)
                     t_start = time.time()
                     self.session.images[url] = self.session.fetch(url)
@@ -144,7 +148,7 @@ class FKLIMGSucker(IMGSucker):
                     continue
                 # TODO: figure the re-write path
                 # newattributes.append((name, path))
-                if not self.session.css.has_key(url):
+                if url not in self.session.css:
                     self.ftestcase.logdd('    link: %s ...' % url)
                     t_start = time.time()
                     self.session.css[url] = self.session.fetch(url)
@@ -168,6 +172,71 @@ def WTC_log(self, message, content):
     pass
 WebTestCase.log = WTC_log
 
+def decodeCookies(url, server, headers, cookies):
+    """Decode cookies into the supplied cookies dictionary,
+    according to RFC 6265.
+
+    Relevant specs:
+    http://www.ietf.org/rfc/rfc2109.txt (obsolete)
+    http://www.ietf.org/rfc/rfc2965.txt (obsolete)
+    http://www.ietf.org/rfc/rfc6265.txt (proposed standard)
+    """
+    # see rfc 6265, section 5.1.4
+    # empty path => '/'
+    # path must begin with '/', so we only weed out the rightmost '/'
+    request_path = urlparse.urlparse(url)[2]
+    if len(request_path) > 2 and request_path[-1] == '/':
+        request_path = request_path[:-1]
+    else:
+        request_path = '/'
+
+    # XXX - tried slurping all the set-cookie and joining them on
+    # '\n', some cookies were not parsed. This below worked flawlessly.
+    for ch in headers.getallmatchingheaders('set-cookie'):
+        cookie = Cookie.SimpleCookie(ch.strip()).values()[0]
+
+        # see rfc 6265, section 5.3, step 7
+        path = cookie['path'] or request_path
+
+        # see rfc 6265, section 5.3, step 4 to 6
+        # XXX - we don't bother with cookie persistence
+        # XXX - we don't check for public suffixes
+        if cookie['domain']:
+            domain = cookie['domain']
+            # see rfc6265, section 5.2.3
+            if domain[0] == '.':
+               domain = domain[1:]
+            if not server.endswith(domain):
+               continue
+        else:
+            domain = server
+
+        # all date handling is done is UTC
+        # XXX - need reviewing by someone familiar with python datetime objects
+        now = datetime.datetime.utcnow()
+        expire = datetime.datetime.min
+        maxage = cookie['max-age']
+        # see rfc 6265, section 5.3, step 3
+        if maxage != '':
+            timedelta = int(maxage)
+            if timedelta > 0:
+                expire = now + datetime.timedelta(seconds=timedelta)
+        else:
+            if cookie['expires'] == '':
+                expire = datetime.datetime.max
+            else:
+                expire = datetime.datetime.strptime(cookie['expires'],"%a, %d-%b-%Y %H:%M:%S %Z")
+
+        cookie['expires'] = expire
+        bydom = cookies.setdefault(domain, {})
+        bypath = bydom.setdefault(path, {})
+
+        if expire > now:
+            bypath[cookie.key] = cookie
+        elif cookie.key in bypath:
+            del bypath[cookie.key]
+
+
 # use fl img sucker
 def WTC_pageImages(self, url, page, testcase=None):
     '''Given the HTML page that was loaded from url, grab all the images.
@@ -181,7 +250,7 @@ WebTestCase.pageImages = WTC_pageImages
 
 # WebFetcher fetch
 def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
-             ok_codes=None, key_file=None, cert_file=None, method="GET"):
+             ok_codes=None, key_file=None, cert_file=None, method="GET", consumer=None):
     '''Run a single test request to the indicated url. Use the POST data
     if supplied. Accepts key and certificate file paths for https (ssl/tls)
     connections.
@@ -253,9 +322,19 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
             #raise ValueError, "Can't fetch HTTPS: M2Crypto not installed"
 
         # FL Patch -------------------------
+        try:
+            proxystring = os.environ["https_proxy"].replace("http://", "").replace("https://", "")
+            webproxy['host'] = proxystring.split(":")[0]
+            webproxy['port'] = int(proxystring.split(":")[1])
+        except (KeyError, IndexError, ValueError):
+            webproxy = False
 
         # patched to use the given key and cert file
-        h = httplib.HTTPS(server, int(port), key_file, cert_file)
+        if webproxy:
+            h = httplib.HTTPSConnection(webproxy['host'], webproxy['port'],
+                                        key_file, cert_file)
+        else:
+            h = httplib.HTTPS(server, int(port), key_file, cert_file)
 
         # FL Patch end  -------------------------
 
@@ -264,13 +343,14 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
         else:
             host_header = '%s:%s' % (server, port)
     else:
-        raise ValueError, protocol
+        raise ValueError(protocol)
 
     headers = []
     params = None
     if postdata is not None:
         if webproxy:
-            h.putrequest(method.upper(), "http://%s%s" % (host_header, url))
+            h.putrequest(method.upper(), "%s://%s%s" % (protocol,
+                                                        host_header, url))
         else:
             # Normal post
             h.putrequest(method.upper(), url)
@@ -298,14 +378,17 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
             headers.append(('Content-length', str(len(params))))
     else:
         if webproxy:
-            h.putrequest(method.upper(), "http://%s%s" % (host_header, url))
+            h.putrequest(method.upper(), "%s://%s%s" % (protocol,
+                                                        host_header, url))
         else:
             h.putrequest(method.upper(), url)
 
     # Other Full Request headers
     if self.authinfo:
         headers.append(('Authorization', "Basic %s"%self.authinfo))
-    if not webproxy:
+        
+    #If a value is specified for 'Host' then another value should not be appended
+    if not webproxy and not 'Host' in [k for k, v in self.extra_headers]:
         # HTTPConnection seems to add a host header itself.
         # So we only need to do this if we are not using a proxy.
         headers.append(('Host', host_header))
@@ -323,7 +406,7 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
     cookie_list = []
     for domain, cookies in self.cookies.items():
         # check cookie domain
-        if not server.endswith(domain):
+        if not server.endswith(domain) and domain[1:] != server:
             continue
         for path, cookies in cookies.items():
             # check that the path matches
@@ -361,7 +444,7 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
 
     if self.debug_headers:
         for header in headers:
-            print "Putting header -- %s: %s" % header
+            print("Putting header -- %s: %s" % header)
 
     if params is not None:
         h.send(params)
@@ -372,7 +455,7 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
         errcode = r.status
         errmsg = r.reason
         headers = r.msg
-        if headers is None or headers.has_key('content-length') and headers['content-length'] == "0":
+        if headers is None or 'content-length' in headers and headers['content-length'] == "0":
             data = None
         else:
             data = r.read()
@@ -383,17 +466,28 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
     else:
         # get the body and save it
         errcode, errmsg, headers = h.getreply()
-        if headers is None or headers.has_key('content-length') and headers['content-length'] == "0":
+        if headers is None or 'content-length' in headers and headers['content-length'] == "0":
             response = HTTPResponse(self.cookies, protocol, server, port, url,
                                     errcode, errmsg, headers, None,
                                     self.error_content)
         else:
             f = h.getfile()
             g = cStringIO.StringIO()
-            d = f.read()
+            if consumer is None:
+                d = f.read()
+            else:
+                d = f.readline(1)
             while d:
                 g.write(d)
-                d = f.read()
+                if consumer is None:
+                    d = f.read()
+                else:
+                    ret = consumer(d)
+                    if ret == 0:
+                        # consumer close connection
+                        d = None
+                    else:
+                        d = f.readline(1)
             response = HTTPResponse(self.cookies, protocol, server, port, url,
                                     errcode, errmsg, headers, g.getvalue(),
                                     self.error_content)
@@ -409,7 +503,7 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
     if self.accept_cookies:
         try:
             # decode the cookies and update the cookies store
-            cookie.decodeCookies(url, server, headers, self.cookies)
+            decodeCookies(url, server, headers, self.cookies)
         except:
             if VERBOSE:
                 sys.stdout.write('c')
@@ -424,11 +518,11 @@ def WF_fetch(self, url, postdata=None, server=None, port=None, protocol=None,
                 msg = "Matched error: %s" % content
                 if hasattr(self, 'results') and self.results:
                     self.writeError(url, msg)
-                self.log('Matched error'+`(url, content)`, data)
+                self.log('Matched error'+repr((url, content)), data)
                 if VERBOSE:
                     sys.stdout.write('c')
                     sys.stdout.flush()
-                raise self.failureException, msg
+                raise self.failureException(msg)
 
     if VERBOSE:
         sys.stdout.write('_')
