@@ -25,6 +25,8 @@ Credits goes to Ian Bicking for parsing tcpwatch files.
 
 $Id$
 """
+from __future__ import print_function
+from __future__ import absolute_import
 import os
 import sys
 import re
@@ -34,7 +36,8 @@ from tempfile import mkdtemp
 import rfc822
 from cgi import FieldStorage
 from urlparse import urlsplit
-from utils import truncate, trace, get_version, Data
+from .utils import truncate, trace, get_version, Data
+
 
 def get_null_file():
     if sys.platform.lower().startswith('win'):
@@ -42,16 +45,17 @@ def get_null_file():
     else:
         return "/dev/null"
 
+
 class Request:
     """Store a tcpwatch request."""
     def __init__(self, file_path):
         """Load a tcpwatch request file."""
         self.file_path = file_path
         f = open(file_path, 'rb')
-        line = f.readline().split(None, 2)
+        line = f.readline().replace('\r\r', '\r').split(None, 2)
         if not line:
             trace('# Warning: empty first line on %s\n' % self.file_path)
-            line = f.readline().split(None, 2)
+            line = f.readline().replace('\r\r', '\r').split(None, 2)
         self.method = line[0]
         url = line[1]
         scheme, host, path, query, fragment = urlsplit(url)
@@ -61,7 +65,7 @@ class Request:
         self.path = path
         self.version = line[2].strip()
         self.headers = dict(rfc822.Message(f).items())
-        self.body = f.read()
+        self.body = f.read().replace('\r\r\n','', 1).replace('\r\r', '\r').replace('\r\n', '\n')
         f.close()
 
     def extractParam(self):
@@ -89,11 +93,10 @@ class Request:
             return params
 
         for item in form.list:
-            
             key = item.name
             value = item.value
             filename = item.filename
-            
+
             if filename is None:
                 params.append([key, value])
             else:
@@ -132,7 +135,7 @@ class Response:
             self.status_message = line[2].strip()
         else:
             self.status_message = ''
-        self.headers =  dict(rfc822.Message(f).items())
+        self.headers = dict(rfc822.Message(f).items())
         self.body = f.read()
         f.close()
 
@@ -154,7 +157,7 @@ a FunkLoad script or generates a FunkLoad unit test if test_name is specified.
 
 The default proxy port is 8090.
 
-Note that tcpwatch.py executable must be accessible from your env.
+Note that tcpwatch-httpproxy or tcpwatch.py executable must be accessible from your env.
 
 See http://funkload.nuxeo.org/ for more information.
 
@@ -206,6 +209,9 @@ Examples
                           help="Verbose output")
         parser.add_option("-p", "--port", type="string", dest="port",
                           default=self.port, help="The proxy port.")
+        parser.add_option("-L", type="string", dest="forward",
+                          default=None,
+                          help="Forwarded connection <listen_port>:<dest_host>:<dest_port>.")
         parser.add_option("-i", "--tcp-watch-input", type="string",
                           dest="tcpwatch_path", default=None,
                           help="Path to an existing tcpwatch capture.")
@@ -222,6 +228,7 @@ Examples
         self.verbose = options.verbose
         self.tcpwatch_path = options.tcpwatch_path
         self.port = options.port
+        self.forward = options.forward
         if not test_name and not self.tcpwatch_path:
             self.loop = options.loop
         if test_name:
@@ -236,8 +243,12 @@ Examples
     def startProxy(self):
         """Start a tcpwatch session."""
         self.tcpwatch_path = mkdtemp('_funkload')
-        cmd = self.getTcpWatchCmd() + ' -p %s -s -r %s' % (self.port,
-                                                           self.tcpwatch_path)
+        if self.forward is not None:
+            cmd = self.getTcpWatchCmd() + ' -L %s -s -r %s' % (self.forward,
+                                                               self.tcpwatch_path)
+        else:
+            cmd = self.getTcpWatchCmd() + ' -p %s -s -r %s' % (self.port,
+                                                               self.tcpwatch_path)
         if os.name == 'posix':
             if self.verbose:
                 cmd += ' | grep "T http"'
@@ -269,7 +280,7 @@ Examples
         items.sort()
         return [(v['request'], v['response'])
                 for name, v in items
-                if v.has_key('response')]
+                if 'response' in v]
 
     def extractRequests(self, files):
         """Filter and extract request from tcpwatch files."""
@@ -395,7 +406,7 @@ Examples
         while count:
             count -= 1
             if count:
-                print "Remaining loop: %i" % count
+                print("Remaining loop: %i" % count)
             if self.tcpwatch_path is None:
                 self.startProxy()
             script = self.extractScript()
@@ -406,8 +417,8 @@ Examples
                 self.writeScript(script)
                 self.writeConfiguration()
             else:
-                print script
-                print
+                print(script)
+                print()
             self.tcpwatch_path = None
 
 
