@@ -20,16 +20,19 @@
 
 $Id$
 """
+from __future__ import print_function
+from __future__ import absolute_import
 
 import os
 import sys
 import re
 from commands import getstatusoutput
-from ReportRenderRst import rst_title
-from ReportRenderHtmlBase import RenderHtmlBase
+from .apdex import Apdex
+from .ReportRenderRst import rst_title
+from .ReportRenderHtmlBase import RenderHtmlBase
 from datetime import datetime
-from MonitorPlugins import MonitorPlugins
-from MonitorPluginsDefault import MonitorCPU, MonitorMemFree, MonitorNetwork, MonitorCUs
+from .MonitorPlugins import MonitorPlugins
+from .MonitorPluginsDefault import MonitorCPU, MonitorMemFree, MonitorNetwork, MonitorCUs
 
 def gnuplot(script_path):
     """Execute a gnuplot script."""
@@ -44,7 +47,7 @@ def gnuplot(script_path):
                                os.path.abspath(script_path))
 
     else:
-        cmd = 'cd ' + path + '; gnuplot ' + os.path.abspath(script_path)
+        cmd = 'cd "' + path + '"; gnuplot "' + os.path.abspath(script_path) + '"'
         ret, output = getstatusoutput(cmd)
         if ret != 0:
             raise RuntimeError("Failed to run gnuplot cmd: " + cmd +
@@ -67,7 +70,7 @@ class FakeMonitorConfig:
 class RenderHtmlGnuPlot(RenderHtmlBase):
     """Render stats in html using gnuplot
 
-    Simply render stuff in ReST than ask docutils to build an html doc.
+    Simply render stuff in ReST then ask docutils to build an html doc.
     """
     chart_size = (640, 540)
     #big_chart_size = (640, 480)
@@ -123,7 +126,7 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         cvus = []
         has_error = False
         for cycle in self.cycles:
-            if not stats[cycle].has_key('test'):
+            if 'test' not in stats[cycle]:
                 continue
             values = []
             test = stats[cycle]['test']
@@ -204,9 +207,8 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         lines = ["CUs SPPS ERROR MIN AVG MAX P10 P50 P90 P95 APDEX E G F P U"]
         cvus = []
         has_error = False
-        apdex_t = 0
         for cycle in self.cycles:
-            if not stats[cycle].has_key('page'):
+            if 'page' not in stats[cycle]:
                 continue
             values = []
             page = stats[cycle]['page']
@@ -225,19 +227,16 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
             values.append(str(page.percentiles.perc90))
             values.append(str(page.percentiles.perc95))
             score = page.apdex_score
-            apdex_t = page.apdex.apdex_t
             values.append(str(score))
+
             apdex = ['0', '0', '0', '0', '0']
-            if score < 0.5:
-                apdex[4] = str(score)
-            elif score < 0.7:
-                apdex[3] = str(score)
-            elif score < 0.85:
-                apdex[2] = str(score)
-            elif score < 0.94:
-                apdex[1] = str(score)
-            else:
-                apdex[0] = str(score)
+            score_cls = Apdex.get_score_class(score)
+            score_classes = Apdex.score_classes[:] #copy
+            #flip from worst-to-best to best-to-worst
+            score_classes.reverse()
+            index = score_classes.index(score_cls)
+            apdex[index] = str(score)
+
             values = values + apdex
             lines.append(' '.join(values))
         if len(lines) == 1:
@@ -271,7 +270,7 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         # apdex
         lines.append('set boxwidth 0.8')
         lines.append('set style fill solid .7')
-        lines.append('set ylabel "Apdex %.1f" ' % apdex_t)
+        lines.append('set ylabel "Apdex %.1f" ' % Apdex.T)
         lines.append('set yrange [0:1]')
         lines.append('set key outside top')
         if has_error:
@@ -314,8 +313,109 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         f.close()
         gnuplot(gplot_path)
 
+    def createRPSTimeChart(self):
+        """Create a RPS chart where X-axis represent the time in seconds."""
+        img_path = gnuplot_scriptpath(self.report_dir, 'time_rps.png')
+        plot_path = gnuplot_scriptpath(self.report_dir, 'time_rps.gplot')
+        
+        stats = self.stats
+
+        start_timeline = sys.maxsize
+        end_timeline = -1
+        max_rps = 0
+        min_rps = 0
+        
+        for cycle in self.cycles:
+            dpath = gnuplot_scriptpath(self.report_dir,
+                                       'time_rps-{0}.data'.format(cycle))
+            f = open(dpath, 'w')
+            f.write('Timeline RPS\n')
+
+            try:
+                st = stats[cycle]['response']
+                for k in sorted(st.per_second.iterkeys()):
+                    if k < start_timeline:
+                        start_timeline = k
+                    if k > end_timeline:
+                        end_timeline = k
+
+                    if st.per_second[k] > max_rps:
+                        max_rps = st.per_second[k]
+                        
+                    f.write('{0} {1}\n'.format(k, st.per_second[k]))
+            except Exception as e:
+                print("Exception: {0}".format(e))
+            finally:
+                f.close
+        #print "max rps: {0}".format(max_rps)
+        #print "time range: {0}-{1}".format(start_timeline, end_timeline)
+
+        max_rps = int(max_rps * 1.25)
+
+        f = open(plot_path, "w")
+
+        lines = []
+        lines.append('set output "{0}"'.format(img_path))
+        lines.append('set title "Request Per Second over time"')
+        lines.append('set xlabel "Time line"')
+        lines.append('set xdata time')
+        lines.append('set timefmt "%s"')
+        lines.append('set format x "%H:%M"')
+        lines.append('set ylabel "RPS"')
+        lines.append('set grid')
+        #lines.append('set xrange [{0}:{1}]'.format(0, end_timeline - start_timeline))
+        lines.append('set yrange [{0}:{1}]'.format(min_rps, max_rps))
+        # I don't know why self.getChartSizeTmp() accept cvus which is not used currently.
+        cvus = []
+        lines.append('set terminal png size ' + self.getChartSizeTmp(cvus))
+
+        plot_line = 'plot '
+        colors = [
+            # This RGB value used for the line color for each cycle.
+            # TODO: use more pretty color?
+            "000000",
+            "0000FF",
+            "00FA9A",
+            "191970",
+            "8B008B",
+            "FF00FF",
+            "FFD700",
+            "0000CD",
+            "00BFFF",
+            "00FF00",
+            "7FFF00",
+            "FF0000",
+            "FF8C00",
+                   ];
+        for i, cycle in enumerate(self.cycles):
+            if i != 0:
+                plot_line += ', \\\n'
+            dpath = gnuplot_scriptpath(self.report_dir,
+                                       'time_rps-{0}.data'.format(cycle))
+            #lines.append('set size 1,1\n')
+            #lines.append('set origin 0,0\n')
+            #plot_line += '"' + dpath + '" u ($1 - {0}):($2)'.format(start_timeline)
+            plot_line += '"' + dpath + '" u ($1):($2)'
+            plot_line += ' w linespoints smooth sbezier lw 1 lt 2 lc ' + \
+                         'rgbcolor "#696969" notitle'
+            plot_line += ', \\\n'
+            #plot_line += '"' + dpath + '" u ($1 - {0}):($2)'.format(start_timeline)
+            plot_line += '"' + dpath + '" u ($1):($2)'
+            plot_line += ' w linespoints lw 1 lt 2 lc ' + \
+                         'rgbcolor "#{0}" t "{1} CUs"'.format(colors[i % len(colors)],
+                                                                     stats[cycle]['response'].cvus)
+        lines.append(plot_line)
+        #lines.append('unset multiplot\n')
+        lines = self.fixXLabels('\n'.join(lines) + '\n')
+        f.write(lines)
+        f.close()
+        gnuplot(plot_path)
+        return
+
     def createAllResponseChart(self):
         """Create global responses chart."""
+        self.createRPSTimeChart()
+        
         image_path = gnuplot_scriptpath(self.report_dir, 'requests_rps.png')
         image2_path = gnuplot_scriptpath(self.report_dir, 'requests.png')
         gplot_path = str(os.path.join(self.report_dir, 'requests.gplot'))
@@ -326,7 +426,7 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         cvus = []
         has_error = False
         for cycle in self.cycles:
-            if not stats[cycle].has_key('response'):
+            if 'response' not in stats[cycle]:
                 continue
             values = []
             resp = stats[cycle]['response']
@@ -415,7 +515,7 @@ class RenderHtmlGnuPlot(RenderHtmlBase):
         cvus = []
         has_error = False
         for cycle in self.cycles:
-            if not stats[cycle]['response_step'].has_key(step):
+            if step not in stats[cycle]['response_step']:
                 continue
             values = []
             resp = stats[cycle]['response_step'].get(step)
