@@ -17,12 +17,12 @@
 #
 """Classes that render statistics in emacs org-mode format.
 """
-import os
+from __future__ import absolute_import
 import re
-from ReportRenderRst import RenderRst
-from ReportRenderRst import BaseRst
-import ReportRenderRst
-from MonitorPlugins import MonitorPlugins
+from .ReportRenderRst import RenderRst
+from .ReportRenderRst import BaseRst
+from . import ReportRenderRst
+from .MonitorPlugins import MonitorPlugins
 
 FL_SITE = "http://funkload.nuxeo.org"
 
@@ -78,7 +78,7 @@ BaseRst.sep = '|'
 
 
 class RenderOrg(RenderRst):
-    """Render stats in ReST format."""
+    """Render stats in emacs org-mode format."""
     # number of slowest requests to display
     slowest_items = 5
     with_chart = True
