@@ -19,10 +19,11 @@
 
 $Id$
 """
+from __future__ import absolute_import
 import os
-from shutil import copyfile
-from utils import get_version
-from MonitorPluginsDefault import MonitorCPU, MonitorMemFree, MonitorNetwork, MonitorCUs
+from .utils import get_version
+from .apdex import Apdex
+from .MonitorPluginsDefault import MonitorCPU, MonitorMemFree, MonitorNetwork, MonitorCUs
 
 LI = '*'
 # ------------------------------------------------------------
@@ -40,16 +41,10 @@ def rst_title(title, level=1, newpage=True):
     rst.append('')
     return '\n'.join(rst)
 
-def get_apdex_label(score):
-    if score < 0.5:
-        return "UNACCEPTABLE"
-    if score < 0.7:
-        return "POOR"
-    if score < 0.85:
-        return "FAIR"
-    if score < 0.94:
-        return "Good"
-    return "Excellent"
+def dumb_pluralize(num, word):
+    #Doesn't follow all English rules, but sufficent for our purpose
+    return ' %s %s' % (num, word + ['s',''][num==1])
+
 
 
 class BaseRst:
@@ -139,7 +134,7 @@ class AllResponseRst(BaseRst):
     """AllResponseStat rendering."""
     headers = [ "CUs", "Apdex*", "Rating*", "RPS", "maxRPS", "TOTAL", "SUCCESS","ERROR",
         "MIN", "AVG", "MAX"]
-    image_names = ['requests_rps', 'requests']
+    image_names = ['requests_rps', 'requests', 'time_rps']
     with_apdex = True
 
     def render_stat(self):
@@ -150,7 +145,7 @@ class AllResponseRst(BaseRst):
         ret.append(self.fmt_int % stats.cvus)
         if self.with_apdex:
             ret.append(self.fmt_float % stats.apdex_score)
-            ret.append(self.fmt_str % get_apdex_label(stats.apdex_score))
+            ret.append(self.fmt_str % Apdex.get_label(stats.apdex_score))
         ret.append(self.fmt_float % stats.rps)
         ret.append(self.fmt_float % stats.rps_max)
         ret.append(self.fmt_int % stats.count)
@@ -192,7 +187,7 @@ class ResponseRst(BaseRst):
         ret = [' ' * self.indent]
         ret.append(self.fmt_int % stats.cvus)
         ret.append(self.fmt_float % stats.apdex_score)
-        ret.append(self.fmt_str % get_apdex_label(stats.apdex_score))
+        ret.append(self.fmt_str % Apdex.get_label(stats.apdex_score))
         ret.append(self.fmt_int % stats.count)
         ret.append(self.fmt_int % stats.success)
         ret.append(self.fmt_percent % stats.error_percent)
@@ -250,7 +245,6 @@ class RenderRst:
             self.with_chart = False
         self.date = config['time'][:19].replace('T', ' ')
 
-
     def getRepresentativeCycleStat(self):
         """Return the cycle stat with the maximum number of steps."""
         stats = self.stats
@@ -271,7 +265,7 @@ class RenderRst:
         max_stps = -1
         cycle_r = None
         for cycle in self.cycles:
-            if not stats[cycle].has_key('test'):
+            if 'test' not in stats[cycle]:
                 continue
             stps = stats[cycle]['test'].tps
             if stps > max_stps:
@@ -288,7 +282,7 @@ class RenderRst:
         max_spps = -1
         cycle_r = None
         for cycle in self.cycles:
-            if not stats[cycle].has_key('page'):
+            if 'page' not in stats[cycle]:
                 continue
             if stats[cycle]['page'].apdex_score < 0.85:
                 continue
@@ -338,11 +332,11 @@ class RenderRst:
         self.append(LI + " Target server: %s" % config['server_url'])
         self.append(LI + " Cycles of concurrent users: %s" % config['cycles'])
         self.append(LI + " Cycle duration: %ss" % config['duration'])
-        self.append(LI + " Sleeptime between request: from %ss to %ss" % (
+        self.append(LI + " Sleeptime between requests: from %ss to %ss" % (
             config['sleep_time_min'], config['sleep_time_max']))
-        self.append(LI + " Sleeptime between test case: %ss" %
+        self.append(LI + " Sleeptime between test cases: %ss" %
                     config['sleep_time'])
-        self.append(LI + " Startup delay between thread: %ss" %
+        self.append(LI + " Startup delay between threads: %ss" %
                     config['startup_delay'])
         self.append(LI + " Apdex: |APDEXT|")
         self.append(LI + " FunkLoad_ version: %s" % config['version'])
@@ -361,16 +355,17 @@ class RenderRst:
 
     def renderTestContent(self, test):
         """Render global information about test content."""
-        self.append(rst_title("Bench content", 2)) 
+
+        self.append(rst_title("Bench content", 2))
         config = self.config
         self.append('The test ``%s.%s`` contains: ' % (config['class'],
                                                        config['method']))
         self.append('')
-        self.append(LI + " %s page(s)" % test.pages)
-        self.append(LI + " %s redirect(s)" % test.redirects)
-        self.append(LI + " %s link(s)" % test.links)
-        self.append(LI + " %s image(s)" % test.images)
-        self.append(LI + " %s XML RPC call(s)" % test.xmlrpc)
+        self.append(LI + dumb_pluralize(test.pages, 'page'))
+        self.append(LI + dumb_pluralize(test.redirects, 'redirect'))
+        self.append(LI + dumb_pluralize(test.links, 'link'))
+        self.append(LI + dumb_pluralize(test.images, 'image'))
+        self.append(LI + dumb_pluralize(test.xmlrpc, 'XML-RPC call'))
         self.append('')
 
         self.append('The bench contains:')
@@ -382,25 +377,27 @@ class RenderRst:
         total_responses_error = 0
         stats = self.stats
         for cycle in self.cycles:
-            if stats[cycle].has_key('test'):
+            if 'test' in stats[cycle]:
                 total_tests += stats[cycle]['test'].count
                 total_tests_error += stats[cycle]['test'].error
-            if stats[cycle].has_key('page'):
+            if 'page' in stats[cycle]:
                 stat = stats[cycle]['page']
                 stat.finalize()
                 total_pages += stat.count
                 total_pages_error += stat.error
-            if stats[cycle].has_key('response'):
+            if 'response' in stats[cycle]:
                 total_responses += stats[cycle]['response'].count
                 total_responses_error += stats[cycle]['response'].error
         self.append('')
+        pluralized_t_errs = dumb_pluralize(total_tests_error, 'error')
+        pluralized_p_errs = dumb_pluralize(total_pages_error, 'error')
+        pluralized_r_errs = dumb_pluralize(total_responses_error, 'error')
         self.append(LI + " %s tests" % total_tests + (
-            total_tests_error and ", %s error(s)" % total_tests_error or ''))
+            total_tests_error and "," + pluralized_t_errs or ''))
         self.append(LI + " %s pages" % total_pages + (
-            total_pages_error and ", %s error(s)" % total_pages_error or ''))
+            total_pages_error and "," + pluralized_p_errs or ''))
         self.append(LI + " %s requests" % total_responses + (
-            total_responses_error and ", %s error(s)" %
-            total_responses_error or ''))
+            total_responses_error and "," + pluralized_r_errs or ''))
         self.append('')
 
 
@@ -420,7 +417,7 @@ class RenderRst:
             self.append('')
         renderer = None
         for cycle in self.cycles:
-            if not stats[cycle].has_key(key):
+            if key not in stats[cycle]:
                 continue
             renderer = klass(stats[cycle][key])
             if first:
@@ -503,7 +500,7 @@ class RenderRst:
         self.append(rst_title("Slowest requests", 2))
         cycle = self.getBestCycle()
         cycle_name = None
-        if not (cycle and stats[cycle].has_key('response_step')):
+        if not (cycle and 'response_step' in stats[cycle]):
             return
         steps = stats[cycle]['response_step'].keys()
         items = []
@@ -511,7 +508,7 @@ class RenderRst:
             stat = stats[cycle]['response_step'][step_name]
             stat.finalize()
             items.append((stat.avg, stat.step,
-                          stat.type, stat.url, stat.description, 
+                          stat.type, stat.url, stat.description,
                           stat.apdex_score))
             if not cycle_name:
                 cycle_name = stat.cvus
@@ -523,7 +520,7 @@ class RenderRst:
         for item in items[:number]:
             self.append(LI + ' In page %s, Apdex rating: %s, avg response time: %3.2fs, %s: ``%s``\n'
                         '  `%s`' % (
-                item[1], get_apdex_label(item[5]), item[0], item[2], item[3], item[4]))
+                item[1], Apdex.get_label(item[5]), item[0], item[2], item[3], item[4]))
 
     def renderErrors(self):
         """Render error list."""
@@ -531,7 +528,7 @@ class RenderRst:
             return
         self.append(rst_title("Failures and Errors", 2))
         for status in ('Failure', 'Error'):
-            if not self.error.has_key(status):
+            if status not in self.error:
                 continue
             stats = self.error[status]
             errors = {}
@@ -548,10 +545,11 @@ class RenderRst:
             self.append(rst_title(status + 's', 3))
             for err_type in err_types:
                 stat = errors[err_type][0]
+                pluralized_times = dumb_pluralize(len(errors[err_type]), 'time')
                 if err_type[1]:
-                    self.append(LI + ' %s time(s), code: %s, %s\n'
+                    self.append(LI + '%s, code: %s, %s\n'
                                 '  in %s, line %s: %s' %(
-                        len(errors[err_type]),
+                        pluralized_times,
                         err_type[0],
                         header.get('bobo-exception-type'),
                         err_type[1], err_type[2],
@@ -559,9 +557,9 @@ class RenderRst:
                 else:
                     traceback = stat.traceback and stat.traceback.replace(
                         'File ', '\n    File ') or 'No traceback.'
-                    self.append(LI + ' %s time(s), code: %s::\n\n'
+                    self.append(LI + '%s, code: %s::\n\n'
                                 '    %s\n' %(
-                        len(errors[err_type]),
+                        pluralized_times,
                         err_type[0], traceback))
 
     def renderDefinitions(self):
@@ -569,9 +567,9 @@ class RenderRst:
         self.append(rst_title("Definitions", 2))
         self.append(LI + ' CUs: Concurrent users or number of concurrent threads'
                     ' executing tests.')
-        self.append(LI + ' Request: a single GET/POST/redirect/xmlrpc request.')
+        self.append(LI + ' Request: a single GET/POST/redirect/XML-RPC request.')
         self.append(LI + ' Page: a request with redirects and resource'
-                    ' links (image, css, js) for an html page.')
+                    ' links (image, css, js) for an HTML page.')
         self.append(LI + ' STPS: Successful tests per second.')
         self.append(LI + ' SPPS: Successful pages per second.')
         self.append(LI + ' RPS: Requests per second, successful or not.')
@@ -588,41 +586,8 @@ class RenderRst:
                     ' of pages or requests are delivered.')
         self.append(LI + ' P95: 95th percentile, response time where 95 percent'
                     ' of pages or requests are delivered.')
-        self.append(LI + ''' Apdex T: Application Performance Index, 
-  this is a numerical measure of user satisfaction, it is based
-  on three zones of application responsiveness:
-
-  - Satisfied: The user is fully productive. This represents the
-    time value (T seconds) below which users are not impeded by
-    application response time.
-
-  - Tolerating: The user notices performance lagging within
-    responses greater than T, but continues the process.
-
-  - Frustrated: Performance with a response time greater than 4*T
-    seconds is unacceptable, and users may abandon the process.
-
-    By default T is set to 1.5s this means that response time between 0
-    and 1.5s the user is fully productive, between 1.5 and 6s the
-    responsivness is tolerating and above 6s the user is frustrated.
-
-    The Apdex score converts many measurements into one number on a
-    uniform scale of 0-to-1 (0 = no users satisfied, 1 = all users
-    satisfied).
-
-    Visit http://www.apdex.org/ for more information.''')
-        self.append(LI + ''' Rating: To ease interpretation the Apdex
-  score is also represented as a rating:
-
-  - U for UNACCEPTABLE represented in gray for a score between 0 and 0.5 
-
-  - P for POOR represented in red for a score between 0.5 and 0.7
-
-  - F for FAIR represented in yellow for a score between 0.7 and 0.85
-
-  - G for Good represented in green for a score between 0.85 and 0.94
-
-  - E for Excellent represented in blue for a score between 0.94 and 1.''')
+        self.append(LI + Apdex.description_para)
+        self.append(LI + Apdex.rating_para)
         self.append('')
         self.append('Report generated with FunkLoad_ ' + get_version() +
                     ', more information available on the '
@@ -639,7 +604,7 @@ class RenderRst:
             return '\n'.join(self.rst)
         cycle_r = self.getRepresentativeCycleStat()
 
-        if cycle_r.has_key('test'):
+        if 'test' in cycle_r:
             self.renderTestContent(cycle_r['test'])
 
         self.renderCyclesStat('test', 'Test stats',
@@ -648,15 +613,16 @@ class RenderRst:
         self.renderCyclesStat('page', 'Page stats',
                               'The number of Successful **Pages** Per Second '
                               '(SPPS) over Concurrent Users (CUs).\n'
-                              'Note that an XML RPC call count like a page.')
+                              'Note: an XML-RPC call counts as a page.')
         self.renderCyclesStat('response', 'Request stats',
                               'The number of **Requests** Per Second (RPS) '
-                              'successful or not over Concurrent Users (CUs).')
+                              '(successful or not) over Concurrent Users (CUs).')
         self.renderSlowestRequests(self.slowest_items)
         self.renderMonitors()
         self.renderPageDetail(cycle_r)
         self.renderErrors()
-        self.renderDefinitions()
+        if not self.options.skip_definitions:
+            self.renderDefinitions()
         self.renderHook()
         return '\n'.join(self.rst)
 
