@@ -26,6 +26,8 @@ Similar to unittest.TestProgram but:
 
 $Id: TestRunner.py 24758 2005-08-31 12:33:00Z bdelbosc $
 """
+from __future__ import print_function
+from __future__ import absolute_import
 try:
     import psyco
     psyco.full()
@@ -39,7 +41,7 @@ import unittest
 import re
 from StringIO import StringIO
 from optparse import OptionParser, TitledHelpFormatter
-from utils import red_str, green_str, get_version
+from .utils import red_str, green_str, get_version
 from funkload.FunkLoadTestCase import FunkLoadTestCase
 
 # ------------------------------------------------------------
@@ -77,7 +79,7 @@ else:
         if failures:
             raise self.failureException(self.format_failure(new.getvalue()))
         elif g_doctest_verbose:
-            print new.getvalue()
+            print(new.getvalue())
 
     DocTestCase.runTest = DTC_runTest
 
@@ -109,7 +111,7 @@ class TestLoader(unittest.TestLoader):
                 pass
         for name in dir(module):
             obj = getattr(module, name)
-            if (isinstance(obj, (type, types.ClassType)) and
+            if (isinstance(obj, type) and
                 issubclass(obj, unittest.TestCase)):
                 tests.append(self.loadTestsFromTestCase(obj))
         suite = self.suiteClass(tests)
@@ -130,7 +132,7 @@ class TestLoader(unittest.TestLoader):
         parts = name.split('.')
         if module is None:
             if not parts:
-                raise ValueError, "incomplete test name: %s" % name
+                raise ValueError("incomplete test name: %s" % name)
             else:
                 parts_copy = parts[:]
                 while parts_copy:
@@ -147,91 +149,48 @@ class TestLoader(unittest.TestLoader):
         import unittest
         if type(obj) == types.ModuleType:
             return self.loadTestsFromModule(obj)
-        elif (isinstance(obj, (type, types.ClassType)) and
+        elif (isinstance(obj, type) and
               issubclass(obj, unittest.TestCase)):
             return self.loadTestsFromTestCase(obj)
         elif type(obj) == types.UnboundMethodType:
             # pass funkload options
-            if issubclass(obj.im_class, FunkLoadTestCase):
-                return obj.im_class(obj.__name__, self.options)
+            if issubclass(obj.__self__.__class__, FunkLoadTestCase):
+                return obj.__self__.__class__(obj.__name__, self.options)
             else:
-                return obj.im_class(obj.__name__)
+                return obj.__self__.__class__(obj.__name__)
         elif callable(obj):
             test = obj()
             if not isinstance(test, unittest.TestCase) and \
                not isinstance(test, unittest.TestSuite):
-                raise ValueError, \
-                      "calling %s returned %s, not a test" % (obj,test)
+                raise ValueError("calling %s returned %s, not a test" % (obj,test))
             return test
         else:
-            raise ValueError, "don't know how to make test from: %s" % obj
+            raise ValueError("don't know how to make test from: %s" % obj)
 
 
 
+
+class ColoredStream(object):
+    def __init__(self, stream):
+        self.stream = stream
+
+    def write(self, arg):
+        if arg in ['OK', 'Ok', 'ok', '.']:
+            arg = green_str(arg)
+        elif arg in ['ERROR', 'E', 'FAILED', 'FAIL', 'F']:
+            arg = red_str(arg)
+        sys.stderr.write(arg)
+
+    def __getattr__(self, attr):
+        return getattr(self.stream, attr)
+
 class _ColoredTextTestResult(unittest._TextTestResult):
     """Colored version."""
-    def addSuccess(self, test):
-        unittest.TestResult.addSuccess(self, test)
-        if self.showAll:
-            self.stream.writeln(green_str("Ok"))
-        elif self.dots:
-            self.stream.write(green_str('.'))
-
-    def addError(self, test, err):
-        unittest.TestResult.addError(self, test, err)
-        if self.showAll:
-            self.stream.writeln(red_str("ERROR"))
-        elif self.dots:
-            self.stream.write(red_str('E'))
-
-    def addFailure(self, test, err):
-        unittest.TestResult.addFailure(self, test, err)
-        if self.showAll:
-            self.stream.writeln(red_str("FAIL"))
-        elif self.dots:
-            self.stream.write(red_str('F'))
-
     def printErrorList(self, flavour, errors):
-        for test, err in errors:
-            self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (red_str(flavour),
-                                            self.getDescription(test)))
-            self.stream.writeln(self.separator2)
-            self.stream.writeln("%s" % err)
+        flavour = red_str(flavour)
+        super(_ColoredTextTestResult, self).printErrorList(flavour, errors)
 
 
-class ColoredTextTestRunner(unittest.TextTestRunner):
-    """Override to be color powered."""
-    def _makeResult(self):
-        return _ColoredTextTestResult(self.stream,
-                                      self.descriptions, self.verbosity)
-
-    def run(self, test):
-        "Run the given test case or test suite."
-        result = self._makeResult()
-        startTime = time.time()
-        test(result)
-        stopTime = time.time()
-        timeTaken = float(stopTime - startTime)
-        result.printErrors()
-        self.stream.writeln(result.separator2)
-        run = result.testsRun
-        self.stream.writeln("Ran %d test%s in %.3fs" %
-                            (run, run != 1 and "s" or "", timeTaken))
-        self.stream.writeln()
-        if not result.wasSuccessful():
-            self.stream.write(red_str("FAILED") + " (")
-            failed, errored = map(len, (result.failures, result.errors))
-            if failed:
-                self.stream.write("failures=%d" % failed)
-            if errored:
-                if failed: self.stream.write(", ")
-                self.stream.write("errors=%d" % errored)
-            self.stream.writeln(")")
-        else:
-            self.stream.writeln(green_str("OK"))
-        return result
-
 def filter_testcases(suite, cpattern, negative_pattern=False):
     """Filter a suite with test names that match the compiled regex pattern."""
     new = unittest.TestSuite()
@@ -257,7 +216,7 @@ def display_testcases(suite):
         if isinstance(test, unittest.TestCase):
             name = test.id()
             name = name[1 + name.find('.'):]
-            print name
+            print(name)
         else:
             display_testcases(test)
 
@@ -268,7 +227,7 @@ class TestProgram(unittest.TestProgram):
 
 %prog launch a FunkLoad unit test.
 
-A FunkLoad unittest use a configuration file named [class].conf, this
+A FunkLoad unittest uses a configuration file named [class].conf, this
 configuration is overriden by the command line options.
 
 See http://funkload.nuxeo.org/ for more information.
@@ -316,6 +275,7 @@ Examples
         self.testNames = None
         self.verbosity = 1
         self.color = True
+        self.profile = False
         self.defaultTest = defaultTest
         self.testLoader = testLoader
         self.progName = os.path.basename(argv[0])
@@ -370,6 +330,8 @@ Examples
         global g_doctest_verbose
         parser = OptionParser(self.USAGE, formatter=TitledHelpFormatter(),
                               version="FunkLoad %s" % get_version())
+        parser.add_option("", "--config", type="string", dest="config", metavar='CONFIG',
+                          help="Path to alternative config file.")
         parser.add_option("-q", "--quiet", action="store_true",
                           help="Minimal output.")
         parser.add_option("-v", "--verbose", action="store_true",
@@ -421,6 +383,8 @@ Examples
         parser.add_option("--pause", action="store_true",
                           help="Pause between request, "
                           "press ENTER to continue.")
+        parser.add_option("--profile", action="store_true",
+                          help="Run test under the Python profiler.")
 
         options, args = parser.parse_args()
         if self.module is None:
@@ -445,13 +409,15 @@ Examples
             options.ftest_debug_level = 1
             options.ftest_log_to = 'console file'
             g_doctest_verbose = True
-        else:
-            options.ftest_log_to = 'file'
         if options.debug_level:
             options.ftest_debug_level = int(options.debug_level)
-        self.color = not options.no_color
+        if sys.platform.lower().startswith('win'):
+            self.color = False
+        else:
+            self.color = not options.no_color
         self.test_name_pattern = options.regex
         self.list_tests = options.list
+        self.profile = options.profile
 
         # set testloader options
         self.testLoader.options = options
@@ -465,12 +431,26 @@ Examples
         """Launch the tests."""
         if self.testRunner is None:
             if self.color:
-                self.testRunner = ColoredTextTestRunner(
-                    verbosity=self.verbosity)
+                self.testRunner = unittest.TextTestRunner(
+                    stream =ColoredStream(sys.stderr),
+                    resultclass = _ColoredTextTestResult,
+                    verbosity = self.verbosity)
             else:
                 self.testRunner = unittest.TextTestRunner(
                     verbosity=self.verbosity)
-        result = self.testRunner.run(self.test)
+        if self.profile:
+            import profile, pstats
+            pr = profile.Profile()
+            d = {'self': self}
+            pr.runctx('result = self.testRunner.run(self.test)', {}, d)
+            result = d['result']
+            pr.dump_stats('profiledata')
+            ps = pstats.Stats('profiledata')
+            ps.strip_dirs()
+            ps.sort_stats('cumulative')
+            ps.print_stats()
+        else:
+            result = self.testRunner.run(self.test)
         sys.exit(not result.wasSuccessful())
 
 
