@@ -19,6 +19,7 @@
 
 $Id$
 """
+from __future__ import absolute_import
 import sys, os
 from socket import error as SocketError
 from time import sleep
@@ -28,8 +29,8 @@ from xmlrpclib import ServerProxy
 import logging
 from optparse import OptionParser, TitledHelpFormatter
 
-from utils import create_daemon, get_default_logger, close_logger
-from utils import trace, get_version
+from .utils import get_default_logger, close_logger
+from .utils import trace, get_version
 
 
 def is_server_running(host, port):
@@ -118,7 +119,7 @@ Start %prog XML/RPC daemon.
         if not options.debug:
             trace(' as daemon.\n')
             close_logger(self.server_name)
-            create_daemon()
+            self.create_daemon()
             # re init the logger
             self.logger = get_default_logger(log_to, log_path, level=level,
                                              name=self.server_name)
@@ -195,7 +196,48 @@ Start %prog XML/RPC daemon.
         return "%s running pid = %s" % (self.server_name, os.getpid())
 
 
-
+    # ------------------------------------------------------------
+    # daemon
+    #
+    # See the Chad J. Schroeder example for a full explanation
+    # this version does not chdir to '/' to keep relative path
+    def create_daemon(self):
+        """Detach a process from the controlling terminal and run it in the
+        background as a daemon.
+        """
+        try:
+            pid = os.fork()
+        except OSError as msg:
+            raise Exception("%s [%d]" % (msg.strerror, msg.errno))
+        if (pid == 0):
+            # child
+            os.setsid()
+            try:
+                pid = os.fork()
+            except OSError as msg:
+                raise Exception("%s [%d]" % (msg.strerror, msg.errno))
+            if (pid == 0):
+                # child
+                os.umask(0)
+            else:
+                os._exit(0)
+        else:
+            while (not is_server_running(self.host, self.port)):
+                sleep(1)
+            os._exit(0)
+        import resource
+        maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
+        if (maxfd == resource.RLIM_INFINITY):
+            maxfd = 1024
+        for fd in range(0, maxfd):
+            try:
+                os.close(fd)
+            except OSError:
+                pass
+        os.open('/dev/null', os.O_RDWR)
+        os.dup2(0, 1)
+        os.dup2(0, 2)
+        return(0)
 
 
 # ------------------------------------------------------------
@@ -224,7 +266,8 @@ Execute action on the XML/RPC server.
         self.conf_path = conf_path
         self.port = conf.getint('server', 'port')
         self.url = 'http://%s:%s/' % (self.host, self.port)
-        self.verbose = not options.quiet
+        self.quiet = options.quiet
+        self.verbose = options.verbose
         self.server = ServerProxy(self.url)
 
 
@@ -233,7 +276,9 @@ Execute action on the XML/RPC server.
         parser = OptionParser(self.usage, formatter=TitledHelpFormatter(),
                               version="FunkLoad %s" % get_version())
         parser.add_option("-q", "--quiet", action="store_true",
-                          help="Verbose output")
+                          help="Suppress console output")
+        parser.add_option("-v", "--verbose", action="store_true",
+                          help="Verbose mode (log-level debug)")
         options, args = parser.parse_args(argv)
         if len(args) != 3:
             parser.error("Missing argument")
@@ -241,7 +286,7 @@ Execute action on the XML/RPC server.
 
     def log(self, message, force=False):
         """Log a message."""
-        if force or self.verbose:
+        if force or not self.quiet:
             trace(str(message))
 
     def startServer(self, debug=False):
@@ -249,6 +294,8 @@ Execute action on the XML/RPC server.
         argv = ['cmd', self.conf_path]
         if debug:
             argv.append('-dv')
+        elif self.verbose:
+            argv.append('-v')
         daemon = self.server_class(argv)
         daemon.run()
 
