@@ -8,7 +8,7 @@ from random import random
 from funkload.FunkLoadTestCase import FunkLoadTestCase
 
 class Simple(FunkLoadTestCase):
-    """This test use a configuration file Simple.conf."""
+    """This test uses the configuration file Simple.conf."""
 
     def setUp(self):
         """Setting up test."""
@@ -17,11 +17,11 @@ class Simple(FunkLoadTestCase):
     def test_simple(self):
         # The description should be set in the configuration file
         server_url = self.server_url
-        # begin of test ---------------------------------------------
+        # begin test ---------------------------------------------
         nb_time = self.conf_getInt('test_simple', 'nb_time')
         for i in range(nb_time):
-            self.get(server_url, description='Get url')
-        # end of test -----------------------------------------------
+            self.get(server_url, description='Get URL')
+        # end test -----------------------------------------------
 
 
 if __name__ in ('main', '__main__'):
